define(["require", "exports"], function (require, exports) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.cache30 = exports.cache12 = exports.LRUCache = void 0;
    function nowTime() {
        return Date.now() / 1000;
    }
    var Deque = /** @class */ (function () {
        function Deque() {
            var head = { next: null }, tail = { prev: null };
            head.next = tail;
            tail.prev = head;
            this.head = head;
            this.tail = tail;
        }
        Deque.prototype.push = function (entry) {
            entry.prev = this.head;
            entry.next = this.head.next;
            this.head.next.prev = entry;
            this.head.next = entry;
        };
        Deque.prototype.remove = function (entry) {
            var prev = entry.prev, next = entry.next;
            if (prev) {
                prev.next = next;
            }
            if (next) {
                next.prev = prev;
            }
            delete entry.prev;
            delete entry.next;
        };
        Deque.prototype.use = function (entry) {
            this.remove(entry);
            entry.time = nowTime();
            this.push(entry);
        };
        Deque.prototype.getTail = function () {
            var entry = this.tail.prev;
            if (entry === this.head) {
                return null;
            }
            return entry;
        };
        return Deque;
    }());
    var LRUCache = /** @class */ (function () {
        function LRUCache(timeout) {
            this.timeout = timeout;
            this.store = {};
            this.deque = new Deque();
        }
        LRUCache.prototype.makeEntry = function (key, value) {
            return { key: key, value: value, time: nowTime() };
        };
        LRUCache.prototype.get = function (key) {
            this.sweep();
            var entry = this.store[key];
            if (!entry)
                return null;
            this.deque.use(entry);
            return entry.value;
        };
        LRUCache.prototype.set = function (key, value) {
            this.sweep();
            var entry = this.store[key];
            if (entry) {
                entry.value = value;
                this.deque.use(entry);
            }
            else {
                entry = this.makeEntry(key, value);
                this.store[key] = entry;
                this.deque.push(entry);
            }
        };
        LRUCache.prototype.sweep = function () {
            while (true) {
                var entry = this.deque.getTail();
                if (!entry)
                    return;
                if (nowTime() - entry.time < this.timeout)
                    return;
                this.deque.remove(entry);
                delete this.store[entry.key];
            }
        };
        LRUCache.prototype.setTimeout = function (timeout) {
            this.timeout = timeout;
        };
        return LRUCache;
    }());
    exports.LRUCache = LRUCache;
    // timeout 12h
    exports.cache12 = new LRUCache(12 * 60 * 60);
    // timeout 30m
    exports.cache30 = new LRUCache(30 * 60);
});
