var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
define(["require", "exports", "chord/base/common/objects", "chord/base/common/json", "better-sqlite3", "chord/library/data/parser", "chord/library/data/common"], function (require, exports, objects_1, json_1, Sqlite, parser_1, common_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.LibraryDatabase = void 0;
    var LibraryDatabase = /** @class */ (function () {
        function LibraryDatabase(databasePath) {
            this.db = new Sqlite(databasePath);
        }
        LibraryDatabase.prototype.getDatabase = function () {
            return this.db;
        };
        LibraryDatabase.prototype.getEncryptedPassword = function () {
            var sql = 'SELECT encrypted_password FROM user';
            var result = this.db.prepare(sql).get();
            return result['encrypted_password'];
        };
        LibraryDatabase.prototype.audio = function (songId) {
            var sql = 'SELECT * FROM audio WHERE songId = ?';
            return this.db.prepare(sql).all(songId)
                .map(function (row) { return parser_1.makeAudio(row); });
        };
        LibraryDatabase.prototype.songs = function (songIds) {
            var _this = this;
            var param = '?,'.repeat(songIds.length).slice(0, -1);
            var sql = "SELECT * FROM song WHERE songId IN (" + param + ")";
            return this.db.prepare(sql).all(songIds)
                .map(function (row) { return parser_1.makeSong(row); })
                .map(function (song) {
                song.audios = _this.audio(song.songId);
                return song;
            });
        };
        LibraryDatabase.prototype.libraryItem = function (sql, lastId, size, keyword) {
            if (keyword) {
                var kw = "%" + keyword + "%";
                return this.db.prepare(sql).all({ lastId: lastId, kw: kw, size: size });
            }
            else {
                return this.db.prepare(sql).all({ lastId: lastId, size: size });
            }
        };
        LibraryDatabase.prototype.librarySongs = function (lastId, size, keyword) {
            var _this = this;
            var searchCondition = '';
            if (keyword) {
                searchCondition = '((song.songName like @kw) OR (song.subTitle like @kw) OR (song.albumName like @kw) OR (song.artistName like @kw) OR (song.genres like @kw))';
            }
            var sql = "SELECT song.*, library_song.* FROM library_song INNER JOIN song ON library_song.songId = song.songId WHERE library_song.id < @lastId " + (keyword ? 'AND ' + searchCondition : '') + " ORDER BY library_song.id DESC LIMIT @size";
            return this.libraryItem(sql, lastId, size, keyword)
                .map(function (row) {
                var addAt = row.addAt;
                var id = row.id;
                delete row.id;
                delete row.addAt;
                var song = parser_1.makeSong(row);
                song.audios = _this.audio(song.songId);
                return { id: id, addAt: addAt, song: song };
            });
        };
        LibraryDatabase.prototype.libraryAlbums = function (lastId, size, keyword) {
            var searchCondition = '';
            if (keyword) {
                searchCondition = '((subTitle like @kw) OR (albumName like @kw) OR (artistName like @kw) OR (genres like @kw))';
            }
            var sql = "SELECT * FROM library_album WHERE id < @lastId " + (keyword ? 'AND ' + searchCondition : '') + " ORDER BY id DESC LIMIT @size";
            return this.libraryItem(sql, lastId, size, keyword)
                .map(function (row) {
                var addAt = row.addAt;
                var id = row.id;
                delete row.id;
                delete row.addAt;
                var album = parser_1.makeAlbum(row);
                return { id: id, addAt: addAt, album: album };
            });
        };
        LibraryDatabase.prototype.libraryAlbumSongs = function (albumId) {
            var sql = "SELECT * FROM library_album WHERE albumId = ?";
            var row = this.db.prepare(sql).get(albumId);
            if (row) {
                return this.songs(JSON.parse(row.songs).map(function (row) { delete row.id; return row; }));
            }
            else {
                return [];
            }
        };
        LibraryDatabase.prototype.libraryArtists = function (lastId, size, keyword) {
            var searchCondition = '';
            if (keyword) {
                searchCondition = '((artistName like @kw) OR (genres like @kw))';
            }
            var sql = "SELECT * FROM library_artist WHERE id < @lastId " + (keyword ? 'AND ' + searchCondition : '') + " ORDER BY id DESC LIMIT @size";
            return this.libraryItem(sql, lastId, size, keyword)
                .map(function (row) {
                var addAt = row.addAt;
                var id = row.id;
                delete row.id;
                delete row.addAt;
                var artist = parser_1.makeArtist(row);
                return { id: id, addAt: addAt, artist: artist };
            });
        };
        LibraryDatabase.prototype.libraryCollections = function (lastId, size, keyword) {
            var searchCondition = '';
            if (keyword) {
                searchCondition = '((collectionName like @kw) OR (tags like @kw))';
            }
            var sql = "SELECT * FROM library_collection WHERE id < @lastId " + (keyword ? 'AND ' + searchCondition : '') + " ORDER BY id DESC LIMIT @size";
            return this.libraryItem(sql, lastId, size, keyword)
                .map(function (row) {
                var addAt = row.addAt;
                var id = row.id;
                delete row.id;
                delete row.addAt;
                var collection = parser_1.makeCollection(row);
                return { id: id, addAt: addAt, collection: collection };
            });
        };
        LibraryDatabase.prototype.libraryCollectionSongs = function (collectionId) {
            var sql = 'SELECT * FROM library_collection WHERE collectionId = ?';
            var row = this.db.prepare(sql).get(collectionId);
            if (row) {
                return this.songs(JSON.parse(row.songs).map(function (row) { delete row.id; return row; }));
            }
            else {
                return [];
            }
        };
        LibraryDatabase.prototype.libraryUserProfiles = function (lastId, size, keyword) {
            var searchCondition = '';
            if (keyword) {
                searchCondition = '(userName like @kw)';
            }
            var sql = "SELECT * FROM library_user_profile WHERE id < @lastId " + (keyword ? 'AND ' + searchCondition : '') + " ORDER BY id DESC LIMIT @size";
            return this.libraryItem(sql, lastId, size, keyword)
                .map(function (row) {
                var addAt = row.addAt;
                var id = row.id;
                delete row.id;
                delete row.addAt;
                var userProfile = parser_1.makeUserProfile(row);
                return { id: id, addAt: addAt, userProfile: userProfile };
            });
        };
        LibraryDatabase.prototype.episodes = function (episodeIds) {
            var _this = this;
            var param = '?,'.repeat(episodeIds.length).slice(0, -1);
            var sql = "SELECT * FROM episode WHERE episodeId IN (" + param + ")";
            return this.db.prepare(sql).all(episodeIds)
                .map(function (row) { return parser_1.makeEpisode(row); })
                .map(function (episode) {
                episode.audios = _this.audio(episode.episodeId);
                return episode;
            });
        };
        LibraryDatabase.prototype.libraryEpisodes = function (lastId, size, keyword) {
            var _this = this;
            var searchCondition = '';
            if (keyword) {
                searchCondition = '((episode.episodeName like @kw) OR (episode.subTitle like @kw) OR (episode.podcastName like @kw) OR (episode.radioName like @kw) OR (episode.genres like @kw))';
            }
            var sql = "SELECT episode.*, library_episode.* FROM library_episode INNER JOIN episode ON library_episode.episodeId = episode.episodeId WHERE library_episode.id < @lastId " + (keyword ? 'AND ' + searchCondition : '') + " ORDER BY library_episode.id DESC LIMIT @size";
            return this.libraryItem(sql, lastId, size, keyword)
                .map(function (row) {
                var addAt = row.addAt;
                var id = row.id;
                delete row.id;
                delete row.addAt;
                var episode = parser_1.makeEpisode(row);
                episode.audios = _this.audio(episode.episodeId);
                return { id: id, addAt: addAt, episode: episode };
            });
        };
        LibraryDatabase.prototype.libraryPodcasts = function (lastId, size, keyword) {
            var searchCondition = '';
            if (keyword) {
                searchCondition = '((subTitle like @kw) OR (podcastName like @kw) OR (radioName like @kw) OR (genres like @kw))';
            }
            var sql = "SELECT * FROM library_podcast WHERE id < @lastId " + (keyword ? 'AND ' + searchCondition : '') + " ORDER BY id DESC LIMIT @size";
            return this.libraryItem(sql, lastId, size, keyword)
                .map(function (row) {
                var addAt = row.addAt;
                var id = row.id;
                delete row.id;
                delete row.addAt;
                var podcast = parser_1.makePodcast(row);
                return { id: id, addAt: addAt, podcast: podcast };
            });
        };
        LibraryDatabase.prototype.libraryPodcastEpisodes = function (podcastId) {
            var sql = "SELECT * FROM library_podcast WHERE podcastId = ?";
            var row = this.db.prepare(sql).get(podcastId);
            if (row) {
                return this.episodes(JSON.parse(row.episodes).map(function (row) { delete row.id; return row; }));
            }
            else {
                return [];
            }
        };
        LibraryDatabase.prototype.libraryRadios = function (lastId, size, keyword) {
            var searchCondition = '';
            if (keyword) {
                searchCondition = '(radioName like @kw)';
            }
            var sql = "SELECT * FROM library_radio WHERE id < @lastId " + (keyword ? 'AND ' + searchCondition : '') + " ORDER BY id DESC LIMIT @size";
            return this.libraryItem(sql, lastId, size, keyword)
                .map(function (row) {
                var addAt = row.addAt;
                var id = row.id;
                delete row.id;
                delete row.addAt;
                var radio = parser_1.makeRadio(row);
                return { id: id, addAt: addAt, radio: radio };
            });
        };
        LibraryDatabase.prototype.storeAudio = function (audio, songId) {
            var _audio = __assign({}, audio);
            _audio.songId = songId;
            var columns = Object.keys(_audio);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO audio (" + columnsStr + ") values (" + param + ")";
            this.db.prepare(sql).run(_audio);
            return true;
        };
        // XXX: NO use this
        LibraryDatabase.prototype.addUser = function (username, encrypted_password) {
            var sql = "insert or ignore into user (username, encrypted_password) values (?, ?)";
            this.db.prepare(sql).run(username, encrypted_password);
            return true;
        };
        LibraryDatabase.prototype.storeSong = function (song, addAt) {
            var _this = this;
            // First, store audios
            song.audios.map(function (audio) { return _this.storeAudio(audio, song.songId); });
            var _song = __assign({}, song);
            delete _song.audios;
            objects_1.removeEmtryAttributes(_song);
            parser_1.toNumber(_song);
            json_1.jsonDumpValue(_song);
            var columns = Object.keys(_song);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO song (" + columnsStr + ") VALUES (" + param + ")";
            this.db.prepare(sql).run(_song);
            return true;
        };
        LibraryDatabase.prototype.addSong = function (song, addAt) {
            this.storeSong(song, addAt);
            var param = { addAt: addAt, songId: song.songId };
            var sql = 'INSERT OR IGNORE INTO library_song (songId, addAt) VALUES (@songId, @addAt)';
            var result = this.db.prepare(sql).run(param);
            return { id: result.lastInsertRowid, song: song, addAt: addAt };
        };
        LibraryDatabase.prototype.addAlbum = function (album, addAt) {
            var _this = this;
            // First add song;
            album.songs.map(function (song) { return _this.storeSong(song, addAt); });
            var _album = __assign({}, album);
            _album.songs = album.songs.map(function (song) { return song.songId; });
            objects_1.removeEmtryAttributes(_album);
            parser_1.toNumber(_album);
            json_1.jsonDumpValue(_album);
            _album.addAt = addAt;
            var columns = Object.keys(_album);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO library_album (" + columnsStr + ") VALUES (" + param + ")";
            var result = this.db.prepare(sql).run(_album);
            return { id: result.lastInsertRowid, album: album, addAt: addAt };
        };
        LibraryDatabase.prototype.addArtist = function (artist, addAt) {
            var _artist = __assign({}, artist);
            delete _artist.songs;
            delete _artist.albums;
            objects_1.removeEmtryAttributes(_artist);
            parser_1.toNumber(_artist);
            json_1.jsonDumpValue(_artist);
            _artist.addAt = addAt;
            var columns = Object.keys(_artist);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO library_artist (" + columnsStr + ") VALUES (" + param + ")";
            var result = this.db.prepare(sql).run(_artist);
            return { id: result.lastInsertRowid, artist: artist, addAt: addAt };
        };
        LibraryDatabase.prototype.addCollection = function (collection, addAt) {
            var _this = this;
            // First add song;
            collection.songs.map(function (song) { return _this.storeSong(song, addAt); });
            var _collection = __assign({}, collection);
            _collection.songs = collection.songs.map(function (song) { return song.songId; });
            objects_1.removeEmtryAttributes(_collection);
            parser_1.toNumber(_collection);
            json_1.jsonDumpValue(_collection);
            _collection.addAt = addAt;
            var columns = Object.keys(_collection);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO library_collection (" + columnsStr + ") VALUES (" + param + ")";
            var result = this.db.prepare(sql).run(_collection);
            return { id: result.lastInsertRowid, collection: collection, addAt: addAt };
        };
        LibraryDatabase.prototype.addUserProfile = function (userProfile, addAt) {
            var _userProfile = __assign({}, userProfile);
            delete _userProfile.songs;
            delete _userProfile.artists;
            delete _userProfile.albums;
            delete _userProfile.favoriteCollections;
            delete _userProfile.createdCollections;
            delete _userProfile.followings;
            delete _userProfile.followers;
            objects_1.removeEmtryAttributes(_userProfile);
            parser_1.toNumber(_userProfile);
            json_1.jsonDumpValue(_userProfile);
            _userProfile.addAt = addAt;
            var columns = Object.keys(_userProfile);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO library_user_profile (" + columnsStr + ") VALUES (" + param + ")";
            var result = this.db.prepare(sql).run(_userProfile);
            return { id: result.lastInsertRowid, userProfile: userProfile, addAt: addAt };
        };
        LibraryDatabase.prototype.storeEpisode = function (episode, addAt) {
            var _this = this;
            // First, store audios
            episode.audios.map(function (audio) { return _this.storeAudio(audio, episode.episodeId); });
            var _episode = __assign({}, episode);
            delete _episode.audios;
            objects_1.removeEmtryAttributes(_episode);
            parser_1.toNumber(_episode);
            json_1.jsonDumpValue(_episode);
            var columns = Object.keys(_episode);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO episode (" + columnsStr + ") VALUES (" + param + ")";
            this.db.prepare(sql).run(_episode);
            return true;
        };
        LibraryDatabase.prototype.addEpisode = function (episode, addAt) {
            this.storeEpisode(episode, addAt);
            var param = { addAt: addAt, episodeId: episode.episodeId };
            var sql = 'INSERT OR IGNORE INTO library_episode (episodeId, addAt) VALUES (@episodeId, @addAt)';
            var result = this.db.prepare(sql).run(param);
            return { id: result.lastInsertRowid, episode: episode, addAt: addAt };
        };
        LibraryDatabase.prototype.addPodcast = function (podcast, addAt) {
            var _this = this;
            // First add episode;
            podcast.episodes.map(function (episode) { return _this.storeEpisode(episode, addAt); });
            var _podcast = __assign({}, podcast);
            _podcast.episodes = podcast.episodes.map(function (episode) { return episode.episodeId; });
            objects_1.removeEmtryAttributes(_podcast);
            parser_1.toNumber(_podcast);
            json_1.jsonDumpValue(_podcast);
            _podcast.addAt = addAt;
            var columns = Object.keys(_podcast);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO library_podcast (" + columnsStr + ") VALUES (" + param + ")";
            var result = this.db.prepare(sql).run(_podcast);
            return { id: result.lastInsertRowid, podcast: podcast, addAt: addAt };
        };
        LibraryDatabase.prototype.addRadio = function (radio, addAt) {
            var _radio = __assign({}, radio);
            delete _radio.episodes;
            delete _radio.radios;
            delete _radio.podcasts;
            delete _radio.favoritePodcasts;
            delete _radio.followings;
            delete _radio.followers;
            objects_1.removeEmtryAttributes(_radio);
            parser_1.toNumber(_radio);
            json_1.jsonDumpValue(_radio);
            _radio.addAt = addAt;
            var columns = Object.keys(_radio);
            var columnsStr = columns.join(',');
            var param = columns.map(function (c) { return '@' + c; }).join(',');
            var sql = "INSERT OR IGNORE INTO library_radio (" + columnsStr + ") VALUES (" + param + ")";
            var result = this.db.prepare(sql).run(_radio);
            return { id: result.lastInsertRowid, radio: radio, addAt: addAt };
        };
        LibraryDatabase.prototype.removeSong = function (song) {
            var sql = "DELETE FROM song WHERE songId = @songId AND songId NOT IN (SELECT songId FROM library_song WHERE songId = @songId)";
            this.db.prepare(sql).run({ songId: song.songId });
            return true;
        };
        LibraryDatabase.prototype.deleteSong = function (song) {
            var sql = "DELETE FROM library_song WHERE songId = ?";
            this.db.prepare(sql).run(song.songId);
            // WARN: no remove song from `song` table, that song may be need by album or collection
            // this.removeSong(song);
            return true;
        };
        LibraryDatabase.prototype.deleteAlbum = function (album) {
            var _this = this;
            var sql = "DELETE FROM library_album WHERE albumId = ?";
            this.db.prepare(sql).run(album.albumId);
            album.songs.forEach(function (song) { return _this.removeSong(song); });
            return true;
        };
        LibraryDatabase.prototype.deleteArtist = function (artist) {
            var sql = "DELETE FROM library_artist WHERE artistId = ?";
            this.db.prepare(sql).run(artist.artistId);
            return true;
        };
        LibraryDatabase.prototype.deleteCollection = function (collection) {
            var _this = this;
            var sql = "DELETE FROM library_collection WHERE collectionId = ?";
            this.db.prepare(sql).run(collection.collectionId);
            collection.songs.forEach(function (song) { return _this.removeSong(song); });
            return true;
        };
        LibraryDatabase.prototype.deleteUserProfile = function (userProfile) {
            var sql = "DELETE FROM library_user_profile WHERE userId = ?";
            this.db.prepare(sql).run(userProfile.userId);
            return true;
        };
        LibraryDatabase.prototype.removeEpisode = function (episode) {
            var sql = "DELETE FROM episode WHERE episodeId = @episodeId AND episodeId NOT IN (SELECT episodeId FROM library_episode WHERE episodeId = @episodeId)";
            this.db.prepare(sql).run({ episodeId: episode.episodeId });
            return true;
        };
        LibraryDatabase.prototype.deleteEpisode = function (episode) {
            var sql = "DELETE FROM library_episode WHERE episodeId = ?";
            this.db.prepare(sql).run(episode.episodeId);
            // WARN: no remove episode from `episode` table, that episode may be need by podcast or collection
            // this.removeEpisode(episode);
            return true;
        };
        LibraryDatabase.prototype.deletePodcast = function (podcast) {
            var _this = this;
            var sql = "DELETE FROM library_podcast WHERE podcastId = ?";
            this.db.prepare(sql).run(podcast.podcastId);
            podcast.episodes.forEach(function (episode) { return _this.removeEpisode(episode); });
            return true;
        };
        LibraryDatabase.prototype.deleteRadio = function (radio) {
            var sql = "DELETE FROM library_radio WHERE radioId = ?";
            this.db.prepare(sql).run(radio.radioId);
            return true;
        };
        LibraryDatabase.prototype.exists = function (item) {
            var idName = item.type == 'userProfile' ? 'userId' : item.type + "Id";
            if (!common_1.TABLES[item.type])
                return false;
            var sql = "select 'id' from " + common_1.TABLES[item.type] + " where " + idName + " = ?";
            var result = this.db.prepare(sql).get(item[idName]);
            return !!result;
        };
        return LibraryDatabase;
    }());
    exports.LibraryDatabase = LibraryDatabase;
});
