define(["require", "exports", "chord/library/data/tables/schemes/util"], function (require, exports, util_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.fixTable = exports.makeTable = void 0;
    var TABLE_NAME = 'library_collection';
    var TABLE_COLUMNS = [
        ['id', 'integer primary key asc'],
        ['collectionId', 'text unique on conflict ignore'],
        ['type', 'text'],
        ['origin', 'text'],
        ['collectionOriginalId', 'text'],
        ['url', 'text'],
        ['collectionName', 'text'],
        ['collectionCoverUrl', 'text'],
        ['collectionCoverPath', 'text'],
        ['userId', 'text'],
        ['userMid', 'text'],
        ['userName', 'text'],
        /**
         * ['releaseDate', 'integer']
         * Typo
         *
         * change to integer
         */
        ['releaseDate', 'integer'],
        ['description', 'text'],
        ['tags', 'text'],
        /**
         * ['duration', 'text']
         * Typo
         *
         * change to integer
         */
        ['duration', 'integer'],
        // json array ["songId"]
        ['songs', 'text'],
        /**
         * ['songCount', 'text'],
         * Typo
         *
         * change to integer
         */
        ['songCount', 'integer'],
        ['playCount', 'integer'],
        ['likeCount', 'integer'],
        ['like', 'integer'],
        ['addAt', 'integer'],
    ];
    var ADDONS = null;
    function makeTable(db) {
        var tableSql = util_1.makeTableSql(TABLE_NAME, TABLE_COLUMNS, ADDONS);
        db.exec(tableSql);
    }
    exports.makeTable = makeTable;
    function fixTable(db) {
        util_1.addMissColumns(TABLE_NAME, TABLE_COLUMNS, db);
        // Fix releaseDate, duration, songCount type error
        // {{{
        var tableScheme = db.prepare("PRAGMA table_info(" + TABLE_NAME + ")").all();
        var songCountCol = tableScheme.find(function (columnInfo) { return columnInfo.name == 'songCount' && columnInfo.type == 'text'; });
        if (songCountCol) {
            var tmpTableName = TABLE_NAME + '_' + Date.now();
            // Create a temporary table
            var tableSql = util_1.makeTableSql(tmpTableName, TABLE_COLUMNS, ADDONS);
            db.exec(tableSql);
            var sql = void 0;
            // Update table
            sql = "INSERT INTO " + tmpTableName + " SELECT * FROM " + TABLE_NAME;
            db.exec(sql);
            // Remove old table
            sql = "DROP TABLE " + TABLE_NAME;
            db.exec(sql);
            // Use new table
            sql = "ALTER TABLE " + tmpTableName + " RENAME TO " + TABLE_NAME;
            db.exec(sql);
        }
        // }}}
    }
    exports.fixTable = fixTable;
});
