define(["require", "exports", "chord/base/common/assert"], function (require, exports, assert_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.getUserUrl = exports.getCollectionUrl = exports.getArtistUrl = exports.getAlbumUrl = exports.getSongUrl = exports.getUserId = exports.getCollectionId = exports.getArtistId = exports.getAlbumId = exports.getSongId = exports.OriginUserUrl = exports.OriginCollectionUrl = exports.OriginArtistUrl = exports.OriginAlbumUrl = exports.OriginSongUrl = exports.getOrigin = exports.ORIGIN = void 0;
    var ORIGIN;
    (function (ORIGIN) {
        ORIGIN["xiami"] = "xiami";
        ORIGIN["netease"] = "netease";
        ORIGIN["qq"] = "qq";
        ORIGIN["qianqian"] = "qianqian";
        ORIGIN["ximalaya"] = "ximalaya";
        ORIGIN["himalaya"] = "himalaya";
        ORIGIN["migu"] = "migu";
        ORIGIN["kuwo"] = "kuwo";
    })(ORIGIN = exports.ORIGIN || (exports.ORIGIN = {}));
    function getOrigin(id) {
        assert_1.ok(Array.from(id).filter(function (c) { return c == '|'; }).length == 2, "[ERROR] [getOrigin]: id is not a chord id, " + id);
        var chunk = id.split('|');
        var origin = ORIGIN[chunk[0]];
        var type = chunk[1];
        var originId = chunk[2];
        assert_1.ok(origin, "[ERROR] [getOrigin]: unknown origin: " + origin + ", id: " + id);
        return { origin: origin, type: type, id: originId };
    }
    exports.getOrigin = getOrigin;
    var OriginSongUrl;
    (function (OriginSongUrl) {
        OriginSongUrl["xiami"] = "https://www.xiami.com/song/{id}";
        OriginSongUrl["netease"] = "https://music.163.com/#/song?id={id}";
        OriginSongUrl["qq"] = "https://y.qq.com/n/yqq/song/{id}.html";
        OriginSongUrl["qianqian"] = "http://music.taihe.com/song/{id}";
        OriginSongUrl["migu"] = "http://music.migu.cn/v3/music/song/{id}";
        OriginSongUrl["kuwo"] = "http://kuwo.cn/play_detail/{id}";
    })(OriginSongUrl = exports.OriginSongUrl || (exports.OriginSongUrl = {}));
    var OriginAlbumUrl;
    (function (OriginAlbumUrl) {
        OriginAlbumUrl["xiami"] = "https://www.xiami.com/album/{id}";
        OriginAlbumUrl["netease"] = "https://music.163.com/#/album?id={id}";
        OriginAlbumUrl["qq"] = "https://y.qq.com/n/yqq/album/{id}.html";
        OriginAlbumUrl["qianqian"] = "http://music.taihe.com/album/{id}";
        OriginAlbumUrl["migu"] = "http://music.migu.cn/v3/music/album/{id}";
        OriginAlbumUrl["kuwo"] = "http://kuwo.cn/album_detail/{id}";
    })(OriginAlbumUrl = exports.OriginAlbumUrl || (exports.OriginAlbumUrl = {}));
    var OriginArtistUrl;
    (function (OriginArtistUrl) {
        OriginArtistUrl["xiami"] = "https://www.xiami.com/artist/{id}";
        OriginArtistUrl["netease"] = "https://music.163.com/#/artist?id={id}";
        OriginArtistUrl["qq"] = "https://y.qq.com/n/yqq/singer/{id}.html";
        OriginArtistUrl["qianqian"] = "http://music.taihe.com/artist/{id}";
        OriginArtistUrl["migu"] = "http://music.migu.cn/v3/music/artist/{id}";
        OriginArtistUrl["kuwo"] = "http://kuwo.cn/singer_detail/{id},";
    })(OriginArtistUrl = exports.OriginArtistUrl || (exports.OriginArtistUrl = {}));
    var OriginCollectionUrl;
    (function (OriginCollectionUrl) {
        OriginCollectionUrl["xiami"] = "https://www.xiami.com/collect/{id}";
        OriginCollectionUrl["netease"] = "https://music.163.com/#/playlist?id={id}";
        OriginCollectionUrl["qq"] = "https://y.qq.com/n/yqq/playsquare/{id}.html";
        OriginCollectionUrl["qianqian"] = "http://music.taihe.com/songlist/{id}";
        OriginCollectionUrl["migu"] = "http://music.migu.cn/v3/music/playlist/{id}";
        OriginCollectionUrl["kuwo"] = "http://kuwo.cn/playlist_detail/{id}";
    })(OriginCollectionUrl = exports.OriginCollectionUrl || (exports.OriginCollectionUrl = {}));
    var OriginUserUrl;
    (function (OriginUserUrl) {
        OriginUserUrl["xiami"] = "https://www.xiami.com/u/{id}";
        OriginUserUrl["netease"] = "https://music.163.com/#/user/home?id={id}";
        OriginUserUrl["qq"] = "https://y.qq.com/portal/profile.html?uin={id}";
        // http://music.taihe.com/user?nickname=%E5%91%A8%E6%88%AA%E6%A3%8D%E7%9A%84%E5%8F%8C%E6%9D%B0%E4%BC%A6
        OriginUserUrl["qianqian"] = "http://music.taihe.com/user?nickname={id}";
        OriginUserUrl["migu"] = "http://music.migu.cn/v2/user/{id}";
    })(OriginUserUrl = exports.OriginUserUrl || (exports.OriginUserUrl = {}));
    function getSongId(origin, id) {
        return ORIGIN[origin] + "|song|" + id;
    }
    exports.getSongId = getSongId;
    function getAlbumId(origin, id) {
        return ORIGIN[origin] + "|album|" + id;
    }
    exports.getAlbumId = getAlbumId;
    function getArtistId(origin, id) {
        return ORIGIN[origin] + "|artist|" + id;
    }
    exports.getArtistId = getArtistId;
    function getCollectionId(origin, id) {
        return ORIGIN[origin] + "|collection|" + id;
    }
    exports.getCollectionId = getCollectionId;
    function getUserId(origin, id) {
        return ORIGIN[origin] + "|user|" + id;
    }
    exports.getUserId = getUserId;
    function getSongUrl(origin, id) {
        return OriginSongUrl[origin].replace('{id}', id);
    }
    exports.getSongUrl = getSongUrl;
    function getAlbumUrl(origin, id) {
        return OriginAlbumUrl[origin].replace('{id}', id);
    }
    exports.getAlbumUrl = getAlbumUrl;
    function getArtistUrl(origin, id) {
        return OriginArtistUrl[origin].replace('{id}', id);
    }
    exports.getArtistUrl = getArtistUrl;
    function getCollectionUrl(origin, id) {
        return OriginCollectionUrl[origin].replace('{id}', id);
    }
    exports.getCollectionUrl = getCollectionUrl;
    function getUserUrl(origin, id) {
        return OriginUserUrl[origin].replace('{id}', id);
    }
    exports.getUserUrl = getUserUrl;
});
