var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
define(["require", "exports", "cheerio", "chord/base/browser/htmlContent", "chord/music/common/origin"], function (require, exports, cheerio, htmlContent_1, origin_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.getInfosFromHtml = exports.makeUserProfiles = exports.makeUserProfile = exports.makeArtistAlbums = exports.makeArtists = exports.makeArtist = exports.makeCollections = exports.makeCollection = exports.makeAlbums = exports.makeAlbum = exports.makeLyric = exports.makeSongs = exports.makeSong = exports.makeAudio = void 0;
    var _origin = 'kuwo';
    var _getSongUrl = origin_1.getSongUrl.bind(null, _origin);
    var _getSongId = origin_1.getSongId.bind(null, _origin);
    var _getAlbumUrl = origin_1.getAlbumUrl.bind(null, _origin);
    var _getAlbumId = origin_1.getAlbumId.bind(null, _origin);
    var _getArtistUrl = origin_1.getArtistUrl.bind(null, _origin);
    var _getArtistId = origin_1.getArtistId.bind(null, _origin);
    var _getCollectionUrl = origin_1.getCollectionUrl.bind(null, _origin);
    var _getCollectionId = origin_1.getCollectionId.bind(null, _origin);
    var _getUserUrl = origin_1.getUserUrl.bind(null, _origin);
    var _getUserId = origin_1.getUserId.bind(null, _origin);
    function makeAudio(info) {
        if (!info['url'])
            return null;
        return {
            format: info['type'],
            size: info['size'],
            kbps: Math.floor(info['br'] / 1000),
            url: info['url'],
            path: null,
        };
    }
    exports.makeAudio = makeAudio;
    function makeSong(info) {
        var songOriginalId = info['rid'].toString();
        var albumOriginalId = info['albumid'].toString();
        var albumCoverUrl = info['albumpic'];
        var artistOriginalId = info['artistid'].toString();
        var disable = info['online'] != 1;
        var song = {
            songId: _getSongId(songOriginalId),
            type: 'song',
            origin: _origin,
            songOriginalId: songOriginalId,
            url: _getSongUrl(songOriginalId),
            songName: htmlContent_1.decodeHtml(info['name']),
            albumId: _getAlbumId(albumOriginalId),
            albumOriginalId: albumOriginalId,
            albumName: htmlContent_1.decodeHtml(info['album']),
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(artistOriginalId),
            artistOriginalId: artistOriginalId,
            artistName: htmlContent_1.decodeHtml(info['artist']),
            track: info['track'],
            // millisecond
            duration: info['duration'] * 1000,
            description: info['albuminfo'],
            // millisecond
            releaseDate: Date.parse(info['releaseDate']),
            playCount: 0,
            audios: [],
            disable: disable,
        };
        return song;
    }
    exports.makeSong = makeSong;
    function makeSongs(info) {
        return (info || []).map(function (songInfo) { return makeSong(songInfo); });
    }
    exports.makeSongs = makeSongs;
    function makeLyric(songId, lyricInfo) {
        if (!lyricInfo || lyricInfo.length == 0)
            return null;
        var hasTrans = false;
        var transStart = 0;
        var _a = __read(lyricInfo.slice(lyricInfo.length - 3, lyricInfo.length), 3), a1 = _a[0], a2 = _a[1], e = _a[2];
        if (a1.time == a2.time) {
            hasTrans = true;
            for (var i = 0; i < lyricInfo.length - 1; ++i) {
                if (lyricInfo[i].lineLyric.startsWith('/') || lyricInfo[i].lineLyric.startsWith('/')) {
                    continue;
                }
                if (lyricInfo[i].time == lyricInfo[i + 1].time) {
                    transStart = i - 1;
                    break;
                }
            }
        }
        var chunks = [];
        if (!hasTrans) {
            chunks = lyricInfo.map(function (item) { return ({
                type: 'lyric',
                text: item.lineLyric,
                point: Number.parseFloat(item.time) * 1000,
                translation: null,
            }); });
        }
        else {
            lyricInfo.slice(0, transStart).forEach(function (item) {
                chunks.push({
                    type: 'lyric',
                    text: item.lineLyric,
                    point: Number.parseFloat(item.time) * 1000,
                    translation: null,
                });
            });
            for (var i = 0; i <= (lyricInfo.length - transStart - 1) / 2; ++i) {
                var start = i * 2 + transStart;
                var end = i * 2 + transStart + 2;
                var _b = __read(lyricInfo.slice(start, end), 2), ori = _b[0], trans = _b[1];
                chunks.push({
                    type: 'lyric',
                    text: ori.lineLyric,
                    point: Number.parseFloat(trans.time) * 1000,
                    translation: trans.lineLyric,
                });
            }
        }
        var lyric = {
            songId: _getSongId(songId),
            chunks: chunks,
        };
        return lyric;
    }
    exports.makeLyric = makeLyric;
    function makeAlbum(info) {
        var albumOriginalId = info['albumid'].toString();
        var albumCoverUrl = info['pic'];
        var artistOriginalId = info['artistid'].toString();
        var songs = (info['musicList'] || []).map(function (songInfo) { return makeSong(songInfo); });
        var album = {
            albumId: _getAlbumId(albumOriginalId),
            type: 'album',
            origin: _origin,
            albumOriginalId: albumOriginalId,
            url: _getAlbumUrl(albumOriginalId),
            albumName: htmlContent_1.decodeHtml(info['album']),
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(artistOriginalId),
            artistOriginalId: artistOriginalId,
            artistName: htmlContent_1.decodeHtml(info['artist']),
            description: info['albuminfo'] || null,
            releaseDate: Date.parse(info['releaseDate']),
            songs: songs,
            songCount: info['total'],
        };
        return album;
    }
    exports.makeAlbum = makeAlbum;
    function makeAlbums(info) {
        return (info || []).map(function (albumInfo) { return makeAlbum(albumInfo); });
    }
    exports.makeAlbums = makeAlbums;
    function makeCollection(info) {
        var collectionOriginalId = info['id'].toString();
        var collectionCoverUrl = info['img300'];
        var tags = (info['tag'] || '').split(',').map(function (tag) { return ({ name: tag }); });
        var songs = (info['musicList'] || []).map(function (songInfo) { return makeSong(songInfo); });
        var userOriginalId = info['uname'];
        var userName = htmlContent_1.decodeHtml(info['userName']);
        var collection = {
            collectionId: _getCollectionId(collectionOriginalId),
            type: 'collection',
            origin: _origin,
            collectionOriginalId: collectionOriginalId,
            url: _getCollectionUrl(collectionOriginalId),
            collectionName: htmlContent_1.decodeHtml(info['name']),
            collectionCoverUrl: collectionCoverUrl,
            userId: _getUserId(userOriginalId),
            userName: userName,
            releaseDate: null,
            description: info['info'],
            tags: tags,
            songs: songs,
            songCount: info['total'],
            playCount: info['listencnt'],
        };
        return collection;
    }
    exports.makeCollection = makeCollection;
    function makeCollections(info) {
        return (info || []).map(function (collectionInfo) { return makeCollection(collectionInfo); });
    }
    exports.makeCollections = makeCollections;
    function makeArtist(info) {
        var artistOriginalId = info['id'].toString();
        var artistAvatarUrl = info['pic'];
        var artist = {
            artistId: _getArtistId(artistOriginalId),
            type: 'artist',
            origin: _origin,
            artistOriginalId: artistOriginalId,
            url: _getArtistUrl(artistOriginalId),
            artistName: htmlContent_1.decodeHtml(info['name']),
            // artistAlias: artistAlias,
            artistAvatarUrl: artistAvatarUrl,
            description: info['info'],
            songs: [],
            albums: [],
            playCount: info['playCount'],
        };
        return artist;
    }
    exports.makeArtist = makeArtist;
    function makeArtists(info) {
        return (info || []).map(function (artistInfo) { return makeArtist(artistInfo); });
    }
    exports.makeArtists = makeArtists;
    function makeArtistAlbums(info) {
        return (info['hotAlbums'] || []).map(function (albumInfo) { return makeAlbum(albumInfo); });
    }
    exports.makeArtistAlbums = makeArtistAlbums;
    function makeUserProfile(info, userId) {
        var userOriginalId = userId || info['userId'].toString();
        var user = {
            userId: _getUserId(userOriginalId),
            type: 'userProfile',
            origin: _origin,
            userOriginalId: userOriginalId,
            url: _getUserUrl(userOriginalId),
            userName: htmlContent_1.decodeHtml(info['nickname']),
            userAvatarUrl: info['avatarImg'] || info['avatarUrl'],
            listenCount: info['playCount'],
            followingCount: info['follows'],
            followerCount: info['followeds'],
            favoriteCollectionCount: info['playlistCount'],
            description: info['signature'],
        };
        return user;
    }
    exports.makeUserProfile = makeUserProfile;
    function makeUserProfiles(info) {
        return (info || []).map(function (userProfile) { return makeUserProfile(userProfile); });
    }
    exports.makeUserProfiles = makeUserProfiles;
    function getInfosFromHtml(html) {
        var $ = cheerio.load(html);
        var userOriginalId = /window.hostId\s*=\s*(\d+)/.exec(html)[1];
        var info = {
            userId: _getUserId(userOriginalId),
            followingCount: parseInt($('#follow_count').text().trim()),
            followerCount: parseInt($('#fan_count').text().trim()),
            favoriteCollectionCount: parseInt(/cCount\s*:\s*(\d+)/g.exec(html)[1]),
        };
        return info;
    }
    exports.getInfosFromHtml = getInfosFromHtml;
});
