define(["require", "exports", "chord/music/common/origin", "chord/music/utils/lyric"], function (require, exports, origin_1, lyric_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.makeUserProfiles = exports.makeUserProfile = exports.makeCollections = exports.makeCollection = exports.makeArtists = exports.makeArtist = exports.makeAlbums = exports.makeAlbum = exports.makeSongs = exports.makeSong = exports.makeLyric = exports.makeAudio = void 0;
    var _origin = 'qianqian';
    var _getSongUrl = origin_1.getSongUrl.bind(null, _origin);
    var _getSongId = origin_1.getSongId.bind(null, _origin);
    var _getAlbumUrl = origin_1.getAlbumUrl.bind(null, _origin);
    var _getAlbumId = origin_1.getAlbumId.bind(null, _origin);
    var _getArtistUrl = origin_1.getArtistUrl.bind(null, _origin);
    var _getArtistId = origin_1.getArtistId.bind(null, _origin);
    var _getCollectionUrl = origin_1.getCollectionUrl.bind(null, _origin);
    var _getCollectionId = origin_1.getCollectionId.bind(null, _origin);
    var _getUserUrl = origin_1.getUserUrl.bind(null, _origin);
    var _getUserId = origin_1.getUserId.bind(null, _origin);
    function formatText(str) {
        if (!str)
            return str;
        return str.replace(/<em>|<\/em>/g, '');
    }
    function formatHtml(str) {
        if (!str)
            return str;
        return str.replace(/\n/g, '<br/>');
    }
    function makeAudio(format) {
        var audio = {
            format: format != 'flac' ? 'mp3' : 'flac',
            size: null,
            kbps: format != 'flac' ? Number.parseInt(format) : 720,
            url: null,
        };
        return audio;
    }
    exports.makeAudio = makeAudio;
    function makeLyric(songId, lyricInfo) {
        if (!lyricInfo)
            return null;
        var lyric = lyric_1.makeLyric(lyricInfo);
        lyric.songId = _getSongId(songId);
        return lyric;
    }
    exports.makeLyric = makeLyric;
    function makeSong(info) {
        var songInfo = info['songinfo'] || info;
        var lyricUrl = songInfo['lrclink'];
        var audios = (songInfo['all_rate'] || '').split(',').map(function (a) { return makeAudio(a); }).sort(function (x, y) { return y.kbps - x.kbps; });
        var songOriginalId = songInfo['song_id'].toString();
        var albumOriginalId = songInfo['album_id'].toString();
        var albumName = formatText(songInfo['album_title']);
        var albumCoverUrl = songInfo['pic_small'] ? songInfo['pic_small'].split('@')[0] : null;
        var artistOriginalId = songInfo['artist_id'].toString();
        var artistName = formatText((songInfo['artist'] || songInfo['author']).split(',')[0]);
        var artistMid = songInfo['artist_ting_uid'] || songInfo['ting_uid'];
        var artistAvatarUrl = songInfo['artist_500_500'] ? songInfo['artist_500_500'].split('@')[0] : null;
        var songName = formatText(songInfo['title']);
        var songWriters = songInfo['songwriting'] ? songInfo['songwriting'].split(',').map(function (i) { return formatText(i); }) : null;
        var singers = songInfo['author'] ? songInfo['author'].split(',').map(function (i) { return formatText(i); }) : null;
        var song = {
            songId: _getSongId(songOriginalId),
            type: 'song',
            origin: _origin,
            songOriginalId: songOriginalId,
            url: _getSongUrl(songOriginalId),
            songName: songName,
            songWriters: songWriters,
            singers: singers,
            albumId: _getAlbumId(albumOriginalId),
            albumOriginalId: albumOriginalId,
            albumName: albumName,
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(artistOriginalId),
            artistOriginalId: artistOriginalId,
            artistMid: artistMid,
            artistName: artistName,
            artistAvatarUrl: artistAvatarUrl,
            description: formatHtml(songInfo['info']),
            composer: songInfo['compose'],
            lyricUrl: lyricUrl,
            track: songInfo['album_no'],
            cdSerial: 0,
            // millisecond
            duration: Number.parseInt(songInfo['file_duration']) * 1000,
            // millisecond
            releaseDate: Date.parse(songInfo['publishtime']),
            playCountWeb: songInfo['total_listen_nums'] || null,
            playCount: 0,
            likeCount: songInfo['collect_num'],
            audios: audios,
        };
        return song;
    }
    exports.makeSong = makeSong;
    function makeSongs(info) {
        return (info || []).map(function (songInfo) { return makeSong(songInfo); });
    }
    exports.makeSongs = makeSongs;
    function makeAlbum(info) {
        var albumInfo = info['albumInfo'] || info;
        var albumOriginalId = albumInfo['album_id'].toString();
        var albumName = formatText(albumInfo['title']);
        var albumCoverUrl = albumInfo['pic_small'].split('@')[0];
        var artistOriginalId = albumInfo['artist_id'].toString();
        var artistMid = albumInfo['artist_ting_uid'] || albumInfo['ting_uid'];
        var artistName = formatText(albumInfo['author']);
        var songList = info['songlist'];
        var songs = (songList || []).map(function (song) { return makeSong(song); });
        var album = {
            albumId: _getAlbumId(albumOriginalId),
            type: 'album',
            origin: _origin,
            albumOriginalId: albumOriginalId,
            url: _getAlbumUrl(albumOriginalId),
            albumName: albumName,
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(artistOriginalId),
            artistOriginalId: artistOriginalId,
            artistMid: artistMid,
            artistName: artistName,
            description: formatHtml(albumInfo['info'] || albumInfo['album_desc']),
            releaseDate: Date.parse(albumInfo['publishtime']) || 0,
            company: albumInfo['publishcompany'],
            songs: songs,
            songCount: albumInfo['songs_total'] ? Number.parseInt(albumInfo['songs_total']) : null,
            playCount: albumInfo['listen_num'] ? Number.parseInt(albumInfo['listen_num']) : null,
            likeCount: albumInfo['collect_num'],
        };
        return album;
    }
    exports.makeAlbum = makeAlbum;
    function makeAlbums(info) {
        return (info || []).map(function (albumInfo) { return makeAlbum(albumInfo); });
    }
    exports.makeAlbums = makeAlbums;
    function makeArtist(info) {
        var artistOriginalId = info['artist_id'].toString();
        var artistMid = (info['ting_uid'] || info['artist_ting_uid']).toString();
        var artistAvatarUrl = (info['avatar_small'] || info['avatar_middle'] || info['pic_small']).split('@')[0];
        var artistAlias = formatText(info['nickname']);
        var artistName = formatText(info['name'] || info['author']);
        var artist = {
            artistId: _getArtistId(artistOriginalId),
            type: 'artist',
            origin: _origin,
            artistOriginalId: artistOriginalId,
            url: _getArtistUrl(artistMid),
            artistMid: artistMid,
            artistName: artistName,
            artistAlias: artistAlias ? [artistAlias] : null,
            artistAvatarUrl: artistAvatarUrl,
            area: info['country'],
            description: formatHtml(info['intro'] || info['artist_desc']),
            songs: [],
            albums: [],
            playCount: info['listen_num'] ? Number.parseInt(info['listen_num']) : null,
            likeCount: info['collect_num'],
        };
        return artist;
    }
    exports.makeArtist = makeArtist;
    function makeArtists(info) {
        return (info || []).map(function (artistInfo) { return makeArtist(artistInfo); });
    }
    exports.makeArtists = makeArtists;
    function makeCollection(info) {
        var collInfo = info['info'] || info;
        var collectionOriginalId = (collInfo['list_id'] || collInfo['diy_id']).toString();
        var collectionCoverUrl = collInfo['list_pic_small'] || collInfo['diy_pic'];
        if (collectionCoverUrl)
            collectionCoverUrl = collectionCoverUrl.split('@')[0];
        var tags = (collInfo['list_tag'] || collInfo['diy_tag'] || '').split(',').map(function (tag) { return ({ name: tag }); });
        var collectionName = formatText(collInfo['list_title'] || collInfo['diy_title']);
        var userInfo = collInfo['userinfo'] || collInfo;
        var userId = _getUserId((userInfo['userid'] || userInfo['user_id'] || '').toString());
        var userName = formatText(userInfo['username']);
        var collection = {
            collectionId: _getCollectionId(collectionOriginalId),
            type: 'collection',
            origin: _origin,
            collectionOriginalId: collectionOriginalId,
            url: _getCollectionUrl(collectionOriginalId),
            collectionName: collectionName,
            collectionCoverUrl: collectionCoverUrl,
            userId: userId,
            userName: userName,
            releaseDate: collInfo['createtime'] * 1000,
            description: formatHtml(collInfo['list_desc']),
            tags: tags,
            songs: [],
            songCount: info['song_num'],
            playCount: info['listen_num'],
            likeCount: info['collect_num'],
        };
        return collection;
    }
    exports.makeCollection = makeCollection;
    function makeCollections(info) {
        return (info || []).map(function (collectionInfo) { return makeCollection(collectionInfo); });
    }
    exports.makeCollections = makeCollections;
    function makeUserProfile(info) {
        var userOriginalId = info['userid'].toString();
        var userAvatarUrl = info['userpic'].split('@')[0];
        var userName = formatText(info['username']);
        var countInfo = info['count_info'] || info;
        var followerCount = countInfo['follow_num'];
        var followingCount = countInfo['friend_num'];
        var userProfile = {
            userId: _getUserId(userOriginalId),
            type: 'userProfile',
            origin: _origin,
            userOriginalId: userOriginalId,
            url: _getUserUrl(userName),
            userName: userName,
            userAvatarUrl: userAvatarUrl,
            followerCount: followerCount,
            followingCount: followingCount,
            songCount: info['songFavoriteNum'],
            artistCount: info['artistFavoriteNum'],
            albumCount: info['albumFavoriteNum'],
            favoriteCollectionCount: info['diyFavoriteNum'],
            createdCollectionCount: null,
            description: formatHtml(info['desc']),
        };
        return userProfile;
    }
    exports.makeUserProfile = makeUserProfile;
    function makeUserProfiles(info) {
        return (info || []).map(function (userInfo) { return makeUserProfile(userInfo); });
    }
    exports.makeUserProfiles = makeUserProfiles;
});
