define(["require", "exports", "chord/base/common/checker", "chord/base/browser/htmlContent", "chord/music/common/origin", "chord/music/utils/lyric"], function (require, exports, checker_1, htmlContent_1, origin_1, lyric_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.makeUserProfiles = exports.makeUserProfile = exports.makeCollections = exports.makeEmptyCollection = exports.makeCollection = exports.makeArtists = exports.makeArtist = exports.makeAlbums = exports.makeAlbum = exports.makeLyric = exports.makeSongs = exports.makeSong = exports.FORMAT_MAP = exports.KBPS_MAP = exports.AUDIO_FORMAT_MAP = void 0;
    var _origin = 'qq';
    var _getSongUrl = origin_1.getSongUrl.bind(null, _origin);
    var _getSongId = origin_1.getSongId.bind(null, _origin);
    var _getAlbumUrl = origin_1.getAlbumUrl.bind(null, _origin);
    var _getAlbumId = origin_1.getAlbumId.bind(null, _origin);
    var _getArtistUrl = origin_1.getArtistUrl.bind(null, _origin);
    var _getArtistId = origin_1.getArtistId.bind(null, _origin);
    var _getCollectionUrl = origin_1.getCollectionUrl.bind(null, _origin);
    var _getCollectionId = origin_1.getCollectionId.bind(null, _origin);
    var _getUserUrl = origin_1.getUserUrl.bind(null, _origin);
    var _getUserId = origin_1.getUserId.bind(null, _origin);
    exports.AUDIO_FORMAT_MAP = {
        '96m4a': 'C400',
        '128mp3': 'M500',
        '320mp3': 'M800',
        '1ape': 'A000',
        '1000flac': 'F000',
    };
    exports.KBPS_MAP = {
        '96aac': 96,
        '128': 128,
        '320': 320,
        '128mp3': 128,
        '320mp3': 320,
        // ape file is not accessable, so missing it
        'ape': 1,
        'flac': 1000,
    };
    exports.FORMAT_MAP = {
        '96aac': 'm4a',
        '128': 'mp3',
        '320': 'mp3',
        '128mp3': 'mp3',
        '320mp3': 'mp3',
        'ape': 'ape',
        'flac': 'flac',
    };
    function getQQAlbumCoverUrl(mid) {
        return 'http://y.gtimg.cn/music/photo_new/T002R300x300M000' + mid + '.jpg?max_age=2592000';
    }
    function getQQArtistAvatarUrl(mid) {
        return 'http://y.gtimg.cn/music/photo_new/T001R300x300M000' + mid + '.jpg?max_age=2592000';
    }
    function makeAudios(info) {
        return Object.keys(info)
            .filter(function (key) { return exports.FORMAT_MAP[key.replace(/size_?/, '').toLowerCase()] && key.startsWith('size') && info[key]; })
            .map(function (key) {
            var chunk = key.replace(/size_?/, '').toLowerCase();
            var format = exports.FORMAT_MAP[chunk];
            var kbps = exports.KBPS_MAP[chunk];
            var size = info[key];
            var audio = {
                format: format,
                kbps: kbps,
                size: size,
            };
            return audio;
        })
            .sort(function (x, y) { return y.kbps - x.kbps; });
    }
    function makeSong(info) {
        var songInfo = info['track_info'] || info;
        var otherInfo = info['info'] || [];
        var songOriginalId = songInfo['id'] || songInfo['songid'];
        // filter songs which have not original id
        // e.g. https://y.qq.com/n/yqq/song2/112/0045hAmL0VbG1A.html at #19
        if (!songOriginalId)
            return null;
        songOriginalId = songOriginalId.toString();
        var songMid = songInfo['mid'] || songInfo['songmid'];
        var songMediaMid = songInfo['strMediaMid'] || (songInfo['file'] ? songInfo['file']['media_mid'] : songMid);
        var songName = songInfo['name'] || songInfo['songname'];
        songName = htmlContent_1.decodeHtml(songName);
        var albumOriginalId = songInfo['album'] ? songInfo['album']['id'].toString() : songInfo['albumid'].toString();
        var albumMid = songInfo['album'] ? songInfo['album']['mid'] : songInfo['albummid'];
        var albumName = songInfo['album'] ? songInfo['album']['name'] : songInfo['albumname'];
        albumName = htmlContent_1.decodeHtml(albumName);
        var albumCoverUrl = getQQAlbumCoverUrl(albumMid);
        var artistOriginalId = songInfo['singer'][0]['id'].toString();
        var artistMid = songInfo['singer'][0]['mid'];
        var artistName = songInfo['singer'][0]['name'];
        artistName = htmlContent_1.decodeHtml(artistName);
        var artistAvatarUrl = getQQArtistAvatarUrl(artistMid);
        var track = songInfo['index_album'] || songInfo['belongCD'];
        var cdSerial = (songInfo['index_cd'] || songInfo['cdIdx']) + 1;
        var songWriters = [];
        var songWritersInfo = otherInfo.filter(function (_info) { return _info['作词'] == 'title' && _info['content'][0]; })[0];
        if (songWritersInfo) {
            songWriters = songWritersInfo['content'].map(function (_info) { return _info['value']; });
        }
        var singers = [];
        var singersInfo = otherInfo.filter(function (_info) { return _info['title'] == '演唱者' && _info['content'][0]; })[0];
        if (singersInfo) {
            singers = singersInfo['content'].map(function (_info) { return _info['value']; });
        }
        var composer = '';
        var composerInfo = otherInfo.filter(function (_info) { return _info['title'] == '作曲' && _info['content'][0]; })[0];
        if (composerInfo) {
            composer = composerInfo['content'][0]['value'];
        }
        var genres = [];
        var genresInfo = otherInfo.filter(function (_info) { return _info['title'] == '歌曲流派' && _info['content'][0]; })[0];
        if (genresInfo) {
            genres = genresInfo['content'].map(function (_info) {
                return {
                    id: _info['id'].toString(),
                    name: _info['value'],
                };
            });
        }
        var audios = makeAudios(songInfo['file'] || songInfo);
        var song = {
            songId: _getSongId(songOriginalId),
            type: 'song',
            origin: _origin,
            songOriginalId: songOriginalId,
            songMid: songMid,
            songMediaMid: songMediaMid,
            url: _getSongUrl(songMid),
            songName: songName,
            subTitle: songInfo['subtitle'] || '',
            songWriters: songWriters,
            singers: singers,
            albumId: _getAlbumId(albumOriginalId),
            albumOriginalId: albumOriginalId,
            albumMid: albumMid,
            albumName: albumName,
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(artistOriginalId),
            artistOriginalId: artistOriginalId,
            artistMid: artistMid,
            artistName: artistName,
            artistAvatarUrl: artistAvatarUrl,
            composer: composer,
            track: track,
            cdSerial: cdSerial,
            genres: genres,
            // millisecond
            duration: songInfo['interval'] * 1000,
            releaseDate: Date.parse(songInfo['time_public']) || 0,
            playCount: 0,
            audios: audios,
        };
        return song;
    }
    exports.makeSong = makeSong;
    function makeSongs(info) {
        return (info || []).map(function (songInfo) { return makeSong(songInfo); }).filter(function (song) { return !!song; });
    }
    exports.makeSongs = makeSongs;
    function makeLyric(songId, lyricInfo, transInfo) {
        if (!lyricInfo)
            return null;
        var lyric = lyric_1.makeLyric(lyricInfo);
        var chunksMap = {};
        lyric.chunks.forEach(function (chunk) { return chunksMap[chunk.point] = chunk; });
        if (transInfo) {
            var trans = lyric_1.makeLyric(transInfo);
            trans.chunks.forEach(function (chunk, index) {
                var lyricChunk = chunksMap[chunk.point];
                if (lyricChunk)
                    lyricChunk.translation = chunk.text;
            });
        }
        lyric.songId = _getSongId(songId);
        return lyric;
    }
    exports.makeLyric = makeLyric;
    function makeAlbum(info) {
        var albumOriginalId = (info['id'] || info['albumid'] || info['albumID'] || info['album_id']).toString();
        var albumMid = info['mid'] || info['album_mid'] || info['albumMID'] || info['albummid'];
        var albumName = info['name'] || info['album_name'] || info['albumName'];
        albumName = htmlContent_1.decodeHtml(albumName);
        var albumCoverUrl = getQQAlbumCoverUrl(albumMid);
        var artistOriginalId = info['singerid'] || info['singer_id'] || info['singerID'];
        var artistMid;
        var artistName;
        if (artistOriginalId) {
            artistOriginalId = artistOriginalId.toString();
            artistMid = info['singermid'] || info['singer_mid'] || info['singerMID'];
            artistName = info['singername'] || info['singer_name'] || info['singerName'];
        }
        else {
            if (checker_1.isObject(info['singers'])) {
                var singerInfo = info['singers'][0];
                artistOriginalId = singerInfo['singer_id'].toString();
                artistMid = singerInfo['singer_mid'];
                artistName = singerInfo['singer_name'];
            }
            else {
                throw new Error("[Error] [qq.parser.makeAlbum]: can not parse info: " + JSON.stringify(info));
            }
        }
        artistName = htmlContent_1.decodeHtml(artistName);
        var tags = info['genre'] ? [{ id: null, name: info['genre'] }] : [];
        var releaseDate = Date.parse(info['aDate'] || info['pub_time'] || info['publicTime'] || info['public_time']);
        var songs = (info['list'] || []).map(function (song) { return makeSong(song); }).filter(function (song) { return !!song; });
        var album = {
            albumId: _getAlbumId(albumOriginalId),
            type: 'album',
            origin: _origin,
            albumOriginalId: albumOriginalId,
            albumMid: albumMid,
            url: _getAlbumUrl(albumMid),
            albumName: albumName,
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(artistOriginalId),
            artistOriginalId: artistOriginalId,
            artistMid: artistMid,
            artistName: artistName,
            tags: tags,
            description: info['desc'] || null,
            releaseDate: releaseDate,
            company: info['company'] || null,
            songs: songs,
            songCount: info['total_song_num'] || info['song_count'],
        };
        return album;
    }
    exports.makeAlbum = makeAlbum;
    function makeAlbums(info) {
        return (info || []).map(function (albumInfo) { return makeAlbum(albumInfo); });
    }
    exports.makeAlbums = makeAlbums;
    function makeArtist(info) {
        var artistOriginalId = (info['singer_id'] || info['id'] || info['singerid']).toString();
        var artistMid = info['singer_mid'] || info['mid'] || info['singermid'];
        var artistName = info['singer_name'] || info['name'] || info['singername'];
        artistName = htmlContent_1.decodeHtml(artistName);
        var artistAvatarUrl = getQQArtistAvatarUrl(artistMid);
        var likeCount = info['num'];
        var artist = {
            artistId: _getArtistId(artistOriginalId),
            type: 'artist',
            origin: _origin,
            artistOriginalId: artistOriginalId,
            artistMid: artistMid,
            url: _getArtistUrl(artistMid),
            artistName: artistName,
            artistAvatarUrl: artistAvatarUrl,
            description: info['SingerDesc'],
            songs: [],
            albums: [],
            likeCount: likeCount,
        };
        return artist;
    }
    exports.makeArtist = makeArtist;
    function makeArtists(info) {
        return (info || []).map(function (artistInfo) { return makeArtist(artistInfo); });
    }
    exports.makeArtists = makeArtists;
    function makeCollection(info) {
        var collectionOriginalId = (info['disstid'] || info['dissid'] || info['tid'] || info['content_id']).toString();
        var collectionCoverUrl = info['logo'] || info['imgurl'] || info['diss_cover'] || info['cover'];
        if (!collectionCoverUrl || !collectionCoverUrl.startsWith('http')) {
            collectionCoverUrl = 'http://y.gtimg.cn/mediastyle/global/img/cover_like.png?max_age=2592000';
        }
        var collectionName = info['dissname'] || info['diss_name'] || info['title'];
        collectionName = htmlContent_1.decodeHtml(collectionName);
        var tags = (info['tags'] || []).map(function (tag) { return ({ id: tag['id'].toString(), name: tag['name'] }); });
        var songs = (info['songlist'] || []).map(function (songInfo) { return makeSong(songInfo); }).filter(function (song) { return !!song; });
        var duration = songs.length != 0 ? songs.map(function (s) { return s.duration; }).reduce(function (x, y) { return x + y; }) : null;
        var userOriginalId;
        var userMid;
        var userName;
        if (info['creator'] && !checker_1.isObject(info['creator'])) {
            userOriginalId = info['creator'];
            userMid = info['creator'];
            userName = info['username'];
        }
        else {
            userOriginalId = info['creator'] ? (info['creator']['creator_uin'] || info['creator']['qq']).toString() : info['uin'];
            userMid = info['creator'] ? info['creator']['encrypt_uin'].toString() : (info['encrypt_uin'] || info['uin']);
            userName = info['creator'] ? info['creator']['name'] : info['nickname'];
        }
        var userId = _getUserId(userOriginalId);
        userName = htmlContent_1.decodeHtml(userName);
        var playCount = info['visitnum'] || info['listennum'] || info['listen_num'];
        var releaseDate = (info['ctime'] * 1000) || typeof (info['createtime']) == 'number' ? info['createtime'] * 1000 : Date.parse(info['createtime']);
        var songCount = info['total_song_num'] || info['song_count'] || info['songnum'] || info['song_cnt'];
        var collection = {
            collectionId: _getCollectionId(collectionOriginalId),
            type: 'collection',
            origin: _origin,
            collectionOriginalId: collectionOriginalId,
            url: _getCollectionUrl(collectionOriginalId),
            collectionName: collectionName,
            collectionCoverUrl: collectionCoverUrl,
            userId: userId,
            userMid: userMid,
            userName: userName,
            // millisecond
            releaseDate: releaseDate,
            description: info['desc'] || info['introduction'],
            tags: tags,
            duration: duration,
            songs: songs,
            songCount: songCount,
            playCount: playCount,
        };
        return collection;
    }
    exports.makeCollection = makeCollection;
    function makeEmptyCollection(collectionOriginalId) {
        var collection = {
            collectionId: _getCollectionId(collectionOriginalId),
            type: 'collection',
            origin: _origin,
            collectionOriginalId: collectionOriginalId,
            url: _getCollectionUrl(collectionOriginalId),
            songs: [],
        };
        return collection;
    }
    exports.makeEmptyCollection = makeEmptyCollection;
    function makeCollections(info) {
        return (info || []).map(function (collectionInfo) { return makeCollection(collectionInfo); });
    }
    exports.makeCollections = makeCollections;
    function makeUserProfile(info) {
        var userInfo = info['creator'] || info;
        var musicInfo = info['mymusic'] || info;
        var createdCollectionsInfo = info['mydiss'] || info;
        // qq number
        // if there is not qq number, using encrypt_uin
        var userOriginalId = (userInfo['uin'] || info['uin'] || info['encrypt_uin']).toString();
        var userProfile = {
            userId: _getUserId(userOriginalId),
            type: 'userProfile',
            origin: _origin,
            userOriginalId: userOriginalId,
            url: _getUserUrl(userOriginalId),
            userName: userInfo['nick'] || info['nick_name'],
            userMid: userInfo['encrypt_uin'] || info['encrypt_uin'],
            userAvatarUrl: userInfo['headpic'] || info['logo'],
            followerCount: info['listen_num'] != undefined ? info['listen_num'] : userInfo['nums']['fansnum'],
            followingCount: info['follow_num'] != undefined ? info['follow_num'] : userInfo['nums']['followusernum'],
            songCount: musicInfo[1] ? musicInfo[1]['num0'] : null,
            artistCount: userInfo['nums'] ? userInfo['nums']['followsingernum'] : null,
            albumCount: musicInfo[1] ? musicInfo[1]['num1'] : null,
            favoriteCollectionCount: musicInfo[1] ? musicInfo[1]['num2'] : null,
            createdCollectionCount: info['songlist_num'] != undefined ? info['songlist_num'] : createdCollectionsInfo['num'],
            description: info['desc'],
        };
        return userProfile;
    }
    exports.makeUserProfile = makeUserProfile;
    function makeUserProfiles(info) {
        return (info || []).map(function (_info) { return makeUserProfile(_info); });
    }
    exports.makeUserProfiles = makeUserProfiles;
});
