define(["require", "exports", "chord/base/common/time"], function (require, exports, time_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.makeLyric = void 0;
    function cleanLyricFormat(text) {
        return text.trim().replace(/<\d+>/g, '');
    }
    function makeLyric(info) {
        var lyricText = info.trim();
        if (!lyricText)
            return { songId: null, chunks: [] };
        var chunks = [];
        var m;
        info.split('\n').forEach(function (row) {
            if (!row.trim())
                return;
            var textr = /(\[.+?\])+(.*)/;
            m = textr.exec(row);
            if (!m)
                return;
            var text = m[2];
            var isText = text.includes('<1>');
            /**
             * For this case (finded at netease):
             *  [00:30.323][02:01.432] ba ba ba ~~~
             */
            var tagr = /\[(.+?)\]/g;
            while (m = tagr.exec(row)) {
                var tag = m[1];
                if (tag == 'x-trans') {
                    if (chunks.length == 0)
                        continue;
                    var chunk = chunks.slice(-1)[0];
                    chunk.translation = cleanLyricFormat(text);
                    continue;
                }
                if (/^\d\d:/.test(tag)) {
                    if (!isText && chunks.length != 0 && chunks.slice(-1)[0].type != 'lyric') {
                        var point = chunks.slice(-1)[0].point;
                        chunks.push({ type: 'breakline', text: null, point: point });
                    }
                    var chunk = {
                        type: isText ? 'text' : 'lyric',
                        text: cleanLyricFormat(text),
                        point: time_1.parseToMillisecond(tag),
                    };
                    chunks.push(chunk);
                }
            }
        });
        chunks = chunks.sort(function (x, y) { return x.point - y.point; });
        chunks.push({
            type: 'breakline',
            text: null,
            point: chunks.length >= 1 ? chunks.slice(-1)[0].point + 10 : 0
        });
        return { songId: null, chunks: chunks };
    }
    exports.makeLyric = makeLyric;
});
