define(["require", "exports", "howler", "chord/base/common/assert", "chord/unity/api/blocked", "chord/base/browser/proxy"], function (require, exports, howler_1, assert_1, blocked_1, proxy_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.CAudio = void 0;
    /**
     * Global Audio Controller
     *
     * Chord has only one global Audio Controller
     * The class handles ALL audio behaves
     */
    var Audio = /** @class */ (function () {
        function Audio() {
        }
        Audio.onplay = function (soundId, store, url, playItemId) {
            this.onplayFns.forEach(function (item) {
                item.fn(soundId, store, url, playItemId);
            });
        };
        Audio.onpause = function (soundId, store, url, playItemId) {
            this.onpauseFns.forEach(function (item) {
                item.fn(soundId, store, url, playItemId);
            });
        };
        Audio.onend = function (soundId, store, url, playItemId) {
            this.onendFns.forEach(function (item) {
                item.fn(soundId, store, url, playItemId);
            });
        };
        Audio.onseek = function (soundId, store, url, playItemId) {
            this.onseekFns.forEach(function (item) {
                item.fn(soundId, store, url, playItemId);
            });
        };
        Audio.onloaderror = function (soundId, store, url, playItemId) {
            this.onloaderrorFns.forEach(function (item) {
                item.fn(soundId, store, url, playItemId);
            });
        };
        /**
         * Get Global Howl instance
         */
        Audio.getAudio = function () {
            return Audio.audio;
        };
        Audio.hasAudio = function () {
            return !!Audio.audio;
        };
        /**
         * Make a Howl instance
         */
        Audio.doMakeAudio = function (url, playItemId) {
            var audioOptions = {
                src: url,
                // Force to HTML5 so that the audio can stream in (best for large files).
                html5: true,
                autoplay: false,
                rate: Audio.rateVal || 1.0,
                onplay: function (soundId) { return Audio.onplay(soundId, Audio.store, url, playItemId); },
                onpause: function (soundId) { return Audio.onpause(soundId, Audio.store, url, playItemId); },
                onend: function (soundId) { return Audio.onend(soundId, Audio.store, url, playItemId); },
                onseek: function (soundId) { return Audio.onseek(soundId, Audio.store, url, playItemId); },
                onloaderror: function (soundId) { return Audio.onloaderror(soundId, Audio.store, url, playItemId); },
            };
            return new howler_1.Howl(audioOptions);
        };
        /**
         * Make(or change) the Global Howl instance
         */
        Audio.makeAudio = function (url, playItemId) {
            if (Audio.audio) {
                Audio.stop();
                Audio.destroy();
            }
            // Cancel global proxy for China webs
            if (blocked_1.hasChinaDomain(url)) {
                proxy_1.setBrowserGlobalProxy(null);
            }
            Audio.audio = Audio.doMakeAudio(url, playItemId);
        };
        Audio.seek = function (position) {
            assert_1.ok(Audio.audio, '[Audio.seek] no audio instance');
            if (position) {
                Audio.audio.seek(position);
                return position;
            }
            else {
                return Audio.audio.seek(position);
            }
        };
        // Get/set the global volume for all sounds, relative to their own volume.
        Audio.volume = function (value) {
            howler_1.Howler.volume(value);
            return howler_1.Howler.volume();
        };
        Audio.play = function () {
            assert_1.ok(Audio.audio, '[Audio.play] no audio instance');
            Audio.audio.play();
        };
        Audio.stop = function () {
            assert_1.ok(Audio.audio, '[Audio.stop] no audio instance');
            Audio.audio.stop();
        };
        Audio.pause = function () {
            assert_1.ok(Audio.audio, '[Audio.pause] no audio instance');
            Audio.audio.pause();
        };
        // Set and change rate
        Audio.rate = function (r) {
            Audio.rateVal = r;
            if (Audio.audio) {
                Audio.audio.rate(r);
            }
        };
        Audio.playing = function () {
            return !!Audio.audio && Audio.audio.playing();
        };
        /**
         * return unloaded, loading or loaded
         */
        Audio.state = function () {
            assert_1.ok(Audio.audio, '[Audio.state] no audio instance');
            return Audio.audio.state();
        };
        Audio.duration = function () {
            return Audio.audio ? Audio.audio.duration() : 0;
        };
        Audio.destroy = function () {
            if (Audio.audio) {
                // Unload and destroy a Howl object.
                // This will immediately stop all sounds attached to this sound
                // and remove it from the cache.
                Audio.audio.unload();
                delete Audio.audio;
                Audio.audio = null;
            }
        };
        /**
         * register the global redux store
         */
        Audio.registerStore = function (store) {
            Audio.store = store;
        };
        Audio.registerOnPlay = function (name, onplay) {
            Audio.onplayFns = Audio.onplayFns.filter(function (hook) { return hook.name != name; });
            Audio.onplayFns.push({ name: name, fn: onplay });
        };
        Audio.registerOnPause = function (name, onpause) {
            Audio.onpauseFns = Audio.onpauseFns.filter(function (hook) { return hook.name != name; });
            Audio.onpauseFns.push({ name: name, fn: onpause });
        };
        Audio.registerOnEnd = function (name, onend) {
            Audio.onendFns = Audio.onendFns.filter(function (hook) { return hook.name != name; });
            Audio.onendFns.push({ name: name, fn: onend });
        };
        Audio.registerOnSeek = function (name, onseek) {
            Audio.onseekFns = Audio.onseekFns.filter(function (hook) { return hook.name != name; });
            Audio.onseekFns.push({ name: name, fn: onseek });
        };
        Audio.registerOnLoadError = function (name, onloaderror) {
            Audio.onloaderrorFns = Audio.onloaderrorFns.filter(function (hook) { return hook.name != name; });
            Audio.onloaderrorFns.push({ name: name, fn: onloaderror });
        };
        Audio.onplayFns = [];
        Audio.onpauseFns = [];
        Audio.onendFns = [];
        Audio.onseekFns = [];
        Audio.onloaderrorFns = [];
        return Audio;
    }());
    exports.CAudio = Audio;
});
