define(["require", "exports", "chord/workbench/api/utils/playItem", "chord/workbench/api/common/state/notification"], function (require, exports, playItem_1, notification_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.noticeInfo = exports.noticeServerFail = exports.noticeSwitchKbps = exports.noticeBlockedKbps = exports.loginFail = exports.loginSuccess = exports.noticePlayItem = exports.notice = void 0;
    function makeNotice(type, item, message) {
        if (message === void 0) { message = null; }
        return { type: type, item: item, message: message };
    }
    function notice(type, item, message) {
        if (message === void 0) { message = null; }
        var act = {
            type: 'c:notification:notice',
            act: 'c:notification:notice',
            notice: makeNotice(type, item, message),
        };
        var store = window.store;
        process.nextTick(function () { return store.dispatch(act); });
    }
    exports.notice = notice;
    /**
     * Notice playing item
     *
     * item is one TPlayItem | IAlbum | IArtist | ICollection | IUserProfile
     */
    function noticePlayItem(item, count, missing) {
        if (item['type'] == 'song' || item['type'] == 'episode') {
            if (!playItem_1.hasPlayItemAudio(item)) {
                notice(notification_1.NOTICES.NO_AUDIO, item);
            }
            else {
                notice(notification_1.NOTICES.PLAY_SONG, item);
            }
        }
        else {
            if (count == 0) {
                notice(notification_1.NOTICES.NO_SONG, item);
            }
            else if (missing == count) {
                notice(notification_1.NOTICES.NO_SONG, item);
            }
            else if (missing > 0) {
                notice(notification_1.NOTICES.FILTER_SONGS, item, "" + missing);
            }
            else {
                notice(notification_1.NOTICES['PLAY_' + item['type'].toUpperCase()], item);
            }
        }
    }
    exports.noticePlayItem = noticePlayItem;
    function loginSuccess(userProfile) {
        notice(notification_1.NOTICES.LOGIN_SUCCESS, userProfile);
    }
    exports.loginSuccess = loginSuccess;
    function loginFail(origin, message) {
        notice(notification_1.NOTICES.LOGIN_FAIL, origin, message);
    }
    exports.loginFail = loginFail;
    function noticeBlockedKbps(item, kbps) {
        notice(notification_1.NOTICES.BLOCKED_KBPS, item, kbps.toString());
    }
    exports.noticeBlockedKbps = noticeBlockedKbps;
    function noticeSwitchKbps(item, kbps) {
        notice(notification_1.NOTICES.SWITCH_KBPS, item, kbps.toString());
    }
    exports.noticeSwitchKbps = noticeSwitchKbps;
    function noticeServerFail(item) {
        notice(notification_1.NOTICES.SERVER_FAIL, item);
    }
    exports.noticeServerFail = noticeServerFail;
    function noticeInfo(item, message) {
        notice(notification_1.NOTICES.INFO, item, message);
    }
    exports.noticeInfo = noticeInfo;
});
